package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * 压力模式：存在3个不同的地址段，需要发发送4条命令
 *
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 20:44
 */
@Getter
@Setter
@SuperBuilder
public class PressureMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 8128429933652613769L;

    /**
     * 压力源
     */
    @Builder.Default
    private PresourceEnum presource = PresourceEnum.OIL;
    /**
     * 压力触发类型
     */
    @Builder.Default
    private TriggerTypeEnum triggerType = TriggerTypeEnum.L0;

    /**
     * 开井压力：单位Mpa，*100，[0~9999]
     */
    @Builder.Default
    private int pressureOpen = 0;

    /**
     * 关井压力： 单位Mpa，*100，[0~9999]
     */
    @Builder.Default
    private int pressureClose = 0;

    /**
     * 传感器压力稳定时间：秒，[0~36000]
     */
    @Builder.Default
    private int stabilityTime = 0;

    /**
     * 最大开井时长
     */
    @Builder.Default
    private String maxOpenWell = "00:00:00";
    /**
     * 最小开井时长
     */
    @Builder.Default
    private String minOpenWell = "00:00:00";
    /**
     * 最大关井时长
     */
    @Builder.Default
    private String maxCloseWell = "00:00:00";
    /**
     * 最小关井时长
     */
    @Builder.Default
    private String minCloseWell = "00:00:00";
    /**
     * 开井压力保护使能
     */
    @Builder.Default
    private boolean presproTect = false;
    /**
     * 开井压力保护源
     */
    @Builder.Default
    private PresourceEnum presproSource = PresourceEnum.OIL;
    /**
     * 开井压力限制上限，[0-9999]
     */
    @Builder.Default
    private int openWellLimitMax = 0;
    /**
     * 开井压力限制下限，[0-9999]
     */
    @Builder.Default
    private int openWellLimitMin = 0;


    public PressureMode() {
        super(3);
        this.setCode(Command.KNPCV1_RUN_PRESSURE);
    }

    @Override
    public boolean validate() {
        return super.validate()
                && Objects.nonNull(this.presource)
                && Objects.nonNull(this.triggerType)
                && Objects.nonNull(this.presproSource)
                && StringUtils.isNotBlank(this.maxOpenWell)
                && StringUtils.isNotBlank(this.minOpenWell)
                && StringUtils.isNotBlank(this.maxCloseWell)
                && StringUtils.isNotBlank(this.minCloseWell)
                && this.pressureOpen >= 0 && this.pressureOpen <= 9999
                && this.pressureClose >= 0 && this.pressureClose <= 9999
                && this.openWellLimitMax >= 0 && this.openWellLimitMax <= 9999
                && this.openWellLimitMin >= 0 && this.openWellLimitMin <= 9999
                && this.stabilityTime >= 0 && this.stabilityTime <= 36000;
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        List<ModbusCommandDto> commandList = new ArrayList<>(4);
        StringBuilder command = new StringBuilder(150);
        //                                 0005*2->hex
        // 地址码  功能码  起始地址  连续长度   连续字长
        // 01     10     00A0     0005     0A
        command.append("011000A000050A");
        command.append(StringUtils.leftPad(Integer.toHexString(this.presource.getCode()), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureOpen), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureClose), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.triggerType.getCode()), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.stabilityTime), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //                                 000C*2->hex
        // 地址码  功能码  起始地址  连续长度   连续字长
        // 01     10     00AA     000C     18
        command.append("011000AA000C18");
        command.append(this.toHexString(this.maxOpenWell));
        command.append(this.toHexString(this.minOpenWell));
        command.append(this.toHexString(this.maxCloseWell));
        command.append(this.toHexString(this.minCloseWell));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //                                 0004*2->hex
        // 地址码  功能码  起始地址  连续长度   连续字长
        // 01     10     0140     0004     08
        command.append("01100140000408");
        command.append("000").append(this.presproTect ? "1" : "0");
        command.append(StringUtils.leftPad(Integer.toHexString(this.presproSource.getCode()), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.openWellLimitMax), 4, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.openWellLimitMin), 4, "0"));
        commandList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        return commandList;
    }

    public void setMaxOpenWell(String maxOpenWell) {
        this.maxOpenWell = this.timingValidate(maxOpenWell, "maxOpenWell");
    }

    public void setMinOpenWell(String minOpenWell) {
        this.minOpenWell = this.timingValidate(minOpenWell, "minOpenWell");
    }

    public void setMaxCloseWell(String maxCloseWell) {
        this.maxCloseWell = this.timingValidate(maxCloseWell, "maxCloseWell");
    }

    public void setMinCloseWell(String minCloseWell) {
        this.minCloseWell = this.timingValidate(minCloseWell, "minCloseWell");
    }

    /**
     * 触发压力源：
     * 0：油压
     * 1：套压
     * 2：输压
     * 3：差压
     */
    @Getter
    public enum PresourceEnum {
        OIL(0, "油压"), CAS(1, "套压"), PRE(2, "输压"), DIFF(3, "差压");

        private final int code;
        private final String name;

        PresourceEnum(int code, String name) {
            this.code = code;
            this.name = name;
        }
    }

    /**
     * 压力触发类型：
     * 0：大于开井压力开井、小于关井压力关井
     * 1：大于开井压力开井、大于关井压力关井
     * 2：小于开井压力开井、小于关井压力关井
     * 3：小于开井压力开井、大于关井压力关井
     */
    @Getter
    public enum TriggerTypeEnum {
        L0(0, "大于开井压力开井、小于关井压力关井"),
        L1(1, "大于开井压力开井、大于关井压力关井"),
        L2(2, "小于开井压力开井、小于关井压力关井"),
        L3(3, "小于开井压力开井、大于关井压力关井");

        private final int code;
        private final String name;

        TriggerTypeEnum(int code, String name) {
            this.code = code;
            this.name = name;
        }

    }

}
